@@ -1,13 +1,9 @@
 from __future__ import annotations
 
 import ast
-import sys
-from typing import Any, Generator
-
-if sys.version_info >= (3, 8):
-    from importlib.metadata import version
-else:
-    from importlib_metadata import version
+from importlib.metadata import version
+from typing import Any
+from typing import Generator
 
 
 class ComprehensionChecker:
@@ -42,6 +38,14 @@ class ComprehensionChecker:
         "C415": "C415 Unnecessary subscript reversal of iterable within {func}().",
         "C416": "C416 Unnecessary {type} comprehension - rewrite using {type}().",
         "C417": "C417 Unnecessary use of map - use a {comp} instead.",
+        "C418": (
+            "C418 Unnecessary {type} passed to dict() - "
+            + "remove the outer call to dict()."
+        ),
+        "C419": (
+            "C419 Unnecessary list comprehension passed to {func}() prevents "
+            + "short-circuiting - rewrite as a generator."
+        ),
     }
 
     def run(self) -> Generator[tuple[int, int, str, type[Any]], None, None]:
@@ -88,13 +92,19 @@ class ComprehensionChecker:
                 elif (
                     num_positional_args == 1
                     and isinstance(node.args[0], ast.ListComp)
-                    and node.func.id in ("list", "set")
+                    and node.func.id in ("list", "set", "any", "all")
                 ):
-                    msg_key = {"list": "C411", "set": "C403"}[node.func.id]
+                    msg_key = {
+                        "list": "C411",
+                        "set": "C403",
+                        "any": "C419",
+                        "all": "C419",
+                    }[node.func.id]
+                    msg = self.messages[msg_key].format(func=node.func.id)
                     yield (
                         node.lineno,
                         node.col_offset,
-                        self.messages[msg_key],
+                        msg,
                         type(self),
                     )
 
@@ -116,10 +126,36 @@ class ComprehensionChecker:
                         type(self),
                     )
 
+                elif (
+                    num_positional_args == 1
+                    and num_keyword_args == 0
+                    and isinstance(node.args[0], (ast.Dict, ast.DictComp))
+                    and node.func.id == "dict"
+                ):
+                    if isinstance(node.args[0], ast.Dict):
+                        type_ = "dict"
+                    else:
+                        type_ = "dict comprehension"
+                    yield (
+                        node.lineno,
+                        node.col_offset,
+                        self.messages["C418"].format(type=type_),
+                        type(self),
+                    )
+
                 elif (
                     num_positional_args == 1
                     and isinstance(node.args[0], (ast.Tuple, ast.List))
-                    and node.func.id in ("tuple", "list", "set", "dict")
+                    and (
+                        node.func.id in ("tuple", "list", "set")
+                        or (
+                            node.func.id == "dict"
+                            and all(
+                                isinstance(i, ast.Tuple) and len(i.elts) == 2
+                                for i in node.args[0].elts
+                            )
+                        )
+                    )
                 ):
                     suffix = "rewrite as a {func} literal."
                     msg_key = {
@@ -294,18 +330,31 @@ class ComprehensionChecker:
                             type(self),
                         )
 
-            elif isinstance(node, (ast.ListComp, ast.SetComp)):
+            elif isinstance(node, (ast.DictComp, ast.ListComp, ast.SetComp)):
                 if (
                     len(node.generators) == 1
                     and not node.generators[0].ifs
                     and not node.generators[0].is_async
                     and (
-                        isinstance(node.elt, ast.Name)
-                        and isinstance(node.generators[0].target, ast.Name)
-                        and node.elt.id == node.generators[0].target.id
+                        (
+                            isinstance(node, (ast.ListComp, ast.SetComp))
+                            and isinstance(node.elt, ast.Name)
+                            and isinstance(node.generators[0].target, ast.Name)
+                            and node.elt.id == node.generators[0].target.id
+                        )
+                        or (
+                            isinstance(node, ast.DictComp)
+                            and isinstance(node.key, ast.Name)
+                            and isinstance(node.value, ast.Name)
+                            and isinstance(node.generators[0].target, ast.Tuple)
+                            and len(node.generators[0].target.elts) == 2
+                            and isinstance(node.generators[0].target.elts[0], ast.Name)
+                            and node.generators[0].target.elts[0].id == node.key.id
+                            and isinstance(node.generators[0].target.elts[1], ast.Name)
+                            and node.generators[0].target.elts[1].id == node.value.id
+                        )
                     )
                 ):
-
                     yield (
                         node.lineno,
                         node.col_offset,
