@@ -1,16 +1,11 @@
 from __future__ import annotations
 
 import re
-import sys
+from importlib.metadata import version
 from textwrap import dedent
 
 import pytest
 
-if sys.version_info >= (3, 8):
-    from importlib.metadata import version
-else:
-    from importlib_metadata import version
-
 
 @pytest.fixture
 def flake8_path(flake8_path):
@@ -289,6 +284,8 @@ def test_C405_fail(code, failures, flake8_path):
     "code",
     [
         "foo = dict(range)",
+        "something = (1, 2); dict([something])",
+        "dict([(1,)])",
     ],
 )
 def test_C406_pass(code, flake8_path):
@@ -757,6 +754,10 @@ def test_C415_fail(code, failures, flake8_path):
 @pytest.mark.parametrize(
     "code",
     [
+        "{x, y for x, y, z in zip('abc', '123', 'def')}",
+        "{y: x for x, y in zip('abc', '123')}",
+        "{x: y for x, (y,) in zip('a', ('1',))}",
+        "{x: z for x, (y,), z in zip('a', ('1',), 'b')}",
         "[str(x) for x in range(5)]",
         "[x + 1 for x in range(5)]",
         "[x for x in range(5) if x % 2]",
@@ -785,21 +786,27 @@ def test_C416_pass(code, flake8_path):
     assert result.out_lines == []
 
 
-# Column offset for list comprehensions was incorrect in Python < 3.8.
-# See https://bugs.python.org/issue31241 for details.
-if sys.version_info >= (3, 8):
-    list_comp_col_offset = 0
-else:
-    list_comp_col_offset = 1
-
-
 @pytest.mark.parametrize(
     "code,failures",
     [
+        (
+            "{x: y for x, y in zip(range(5), range(5))}",
+            [
+                "./example.py:1:1: C416 Unnecessary dict comprehension - "
+                + "rewrite using dict().",
+            ],
+        ),
+        (
+            "{x: y for (x, y) in zip(range(5), range(5))}",
+            [
+                "./example.py:1:1: C416 Unnecessary dict comprehension - "
+                + "rewrite using dict().",
+            ],
+        ),
         (
             "[x for x in range(5)]",
             [
-                f"./example.py:1:{1 + list_comp_col_offset}: C416 Unnecessary "
+                "./example.py:1:1: C416 Unnecessary "
                 + "list comprehension - rewrite using list()."
             ],
         ),
@@ -873,3 +880,86 @@ def test_C417_fail(code, failures, flake8_path):
     (flake8_path / "example.py").write_text(dedent(code))
     result = flake8_path.run_flake8()
     assert result.out_lines == failures
+
+
+@pytest.mark.parametrize(
+    "code",
+    [
+        "dict({}, a=1)",
+        "dict({x: 1 for x in range(1)}, a=1)",
+    ],
+)
+def test_C418_pass(code, flake8_path):
+    (flake8_path / "example.py").write_text(dedent(code))
+    result = flake8_path.run_flake8()
+    assert result.out_lines == []
+
+
+@pytest.mark.parametrize(
+    "code,failures",
+    [
+        (
+            "dict({})",
+            [
+                "./example.py:1:1: C418 Unnecessary dict passed to dict() - "
+                + "remove the outer call to dict()."
+            ],
+        ),
+        (
+            "dict({'a': 1})",
+            [
+                "./example.py:1:1: C418 Unnecessary dict passed to dict() - "
+                + "remove the outer call to dict()."
+            ],
+        ),
+        (
+            "dict({'x': 1 for x in range(10)})",
+            [
+                "./example.py:1:1: C418 Unnecessary dict comprehension passed "
+                + "to dict() - remove the outer call to dict()."
+            ],
+        ),
+    ],
+)
+def test_C418_fail(code, failures, flake8_path):
+    (flake8_path / "example.py").write_text(dedent(code))
+    result = flake8_path.run_flake8()
+    assert result.out_lines == failures
+
+
+@pytest.mark.parametrize(
+    "code",
+    [
+        "any(num == 3 for num in range(5))",
+        "all(num == 3 for num in range(5))",
+    ],
+)
+def test_C419_pass(code, flake8_path):
+    (flake8_path / "example.py").write_text(dedent(code))
+    result = flake8_path.run_flake8()
+    assert result.out_lines == []
+
+
+@pytest.mark.parametrize(
+    "code,failures",
+    [
+        (
+            "any([num == 3 for num in range(5)])",
+            [
+                "./example.py:1:1: C419 Unnecessary list comprehension passed "
+                + "to any() prevents short-circuiting - rewrite as a generator."
+            ],
+        ),
+        (
+            "all([num == 3 for num in range(5)])",
+            [
+                "./example.py:1:1: C419 Unnecessary list comprehension passed "
+                + "to all() prevents short-circuiting - rewrite as a generator."
+            ],
+        ),
+    ],
+)
+def test_C419_fail(code, failures, flake8_path):
+    (flake8_path / "example.py").write_text(dedent(code))
+    result = flake8_path.run_flake8()
+    assert result.out_lines == failures
